<?php

class ControllerExtensionPaymentEnot extends Controller
{

    public function index()
    {
        $this->load->model('extension/payment/enot');
        $url = $this->model_extension_payment_enot->getPaymentUrl();

        $data['url'] = $url;
        return $this->load->view('extension/payment/enotredirect', $data);
    }

    public function getMethodList($data)
    {
        $data['payment_enot_load_method_list'] = $this->config->get('payment_enot_load_method_list');
        $data['payment_enot_sort_order'] = $this->config->get('payment_enot_sort_order');
        return $this->load->view('extension/payment/enot', $data);
    }

    public function save()
    {
        if (!empty($this->request->post['tariff'])) {
            $this->session->data['enot']['method_current'] = $this->request->post['tariff'];
            $json = array('success');
        } else $json = array('error');

        $this->response->addHeader('Content-Type: application/json');
        $this->response->setOutput(json_encode($json));
    }

    public function response()
    {
        $this->load->model('checkout/order');

        if (isset($this->request->get['fail']) and $this->request->get['fail']) {
            $this->response->redirect($this->url->link('checkout/confirm', '', true));
        } else {
            $this->cart->clear();
            $this->response->redirect($this->url->link('checkout/success', '', true));
        }
    }

    public function callback()
    {

        if ($_SERVER['REQUEST_METHOD'] !== 'POST') {
            file_put_contents(DIR_LOGS . 'error.log', date('Y-m-d H:i:s') . ' - callback # enot: Invalid method.' . "\r\n", FILE_APPEND);
            http_response_code(405); // Method Not Allowed
            exit;
        }

        $signature_header = $_SERVER['HTTP_X_API_SHA256_SIGNATURE'] ?? '';

        if (!$signature_header) {
            file_put_contents(DIR_LOGS . 'error.log', date('Y-m-d H:i:s') . ' - callback # enot: not signature.' . "\r\n", FILE_APPEND);
            exit;
        }

        $sign = $request_body = file_get_contents('php://input');
        file_put_contents(DIR_LOGS . 'error.log', date('Y-m-d H:i:s') . " - callback # enot: $request_body" . "\r\n", FILE_APPEND);

        $sign = json_decode($sign, true);
        ksort($sign);
        $sign = json_encode($sign);
        $secret_key = $this->config->get('payment_enot_secret_word2');
        $calculated_signature = hash_hmac('sha256', $sign, $secret_key);

        // Compare the calculated signature with the signature in the header
        if (!hash_equals($signature_header, $calculated_signature)) {
            file_put_contents(DIR_LOGS . 'error.log', date('Y-m-d H:i:s') . ' - callback # enot:Invalid signature.' . "\r\n", FILE_APPEND);
            http_response_code(401); // Unauthorized
            exit;
        }
        $request_body = json_decode($request_body, true);

        $order_id = $request_body['order_id'] ?? 0;

        if (!$order_id) {
            file_put_contents(DIR_LOGS . 'error.log', date('Y-m-d H:i:s') . ' - callback # enot: not order_id.' . "\r\n", FILE_APPEND);
            http_response_code(406);
            exit;
        }

        $order_id = explode('_', $request_body['order_id']);
        $order_id = isset($order_id[1]) ? (int)$order_id[1] : 0;

        if (!$order_id) {
            file_put_contents(DIR_LOGS . 'error.log', date('Y-m-d H:i:s') . ' - callback # enot: incorrect order_id.' . "\r\n", FILE_APPEND);
            http_response_code(406);
            exit;
        }

        $status = $request_body['status'] ?? 'unknown';

        if ($status != 'success') {
            file_put_contents(DIR_LOGS . 'error.log', date('Y-m-d H:i:s') . "callback # enot: unsuccessful status for order $order_id - $status \r\n", FILE_APPEND);
            exit;
        }

        $this->load->model('checkout/order');
        $comment = 'Enot transaction id: ' . $request_body['invoice_id'];
        $this->model_checkout_order->addOrderHistory($order_id, $this->config->get('payment_enot_order_status_id'), $comment, $notify = true, $override = false);

    }

}
